@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2008, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -33,9 +31,8 @@
 
 #include "mailutils/libargp.h"
 
-const char *program_version = "GNU dotlock (" PACKAGE_STRING ")";
 static char doc[] =
-N_("GNU dotlock -- lock mail spool files")
+N_("GNU dotlock -- lock mail spool files.")
 "\v"
 N_("Returns 0 on success, 3 if locking the file fails because\
  it's already locked, and 1 if some other kind of error occurred.");
@@ -77,12 +74,12 @@ static int debug;
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
 {
-  static struct mu_argp_node_list lst;
+  static mu_list_t lst;
 
   switch (key)
     {
     case 'd':
-      mu_argp_node_list_new (&lst, "debug", "yes");
+      mu_argp_node_list_new (lst, "debug", "yes");
       break;
 
     case 'u':
@@ -91,11 +88,11 @@ parse_opt (int key, char *arg, struct argp_state *state)
 
     case 'r':
       if (arg)
-	mu_argp_node_list_new (&lst, "retry", arg);
+	mu_argp_node_list_new (lst, "retry", arg);
       break;
 
     case 'f':
-      mu_argp_node_list_new (&lst, "force", arg ? arg : "0");
+      mu_argp_node_list_new (lst, "force", arg ? arg : "0");
       break;
 
     case ARGP_KEY_ARG:
@@ -114,7 +111,7 @@ parse_opt (int key, char *arg, struct argp_state *state)
       break;
       
     case ARGP_KEY_FINI:
-      mu_argp_node_list_finish (&lst, NULL, NULL);
+      mu_argp_node_list_finish (lst, NULL, NULL);
       break;
       
     default:
@@ -137,7 +134,6 @@ struct mu_cfg_param dotlock_cfg_param[] = {
 
 
 const char *dotlock_capa[] = {
-  "license",
   "common",
   "debug",
   NULL
@@ -161,7 +157,7 @@ main (int argc, char *argv[])
 
   argp_err_exit_status = MU_DL_EX_ERROR;
   
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   if (mu_app_init (&argp, dotlock_capa, dotlock_cfg_param, 
 		   argc, argv, 0, NULL, NULL))
     exit (1);
