@@ -1,6 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2008 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006, 2007, 2008, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,10 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA
-*/
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #include <iostream>
 #include <vector>
@@ -43,21 +40,25 @@ main ()
 	Url url (str);
 	url.parse ();
 
-	cout << "\tscheme <" << url.getScheme () << ">" << endl;
-	cout << "\tuser <"   << url.getUser   () << ">" << endl;
-	cout << "\tpasswd <" << url.getPasswd () << ">" << endl;
-	cout << "\tauth <"   << url.getAuth   () << ">" << endl;
-	cout << "\thost <"   << url.getHost   () << ">" << endl;
-	cout << "\tport "    << url.getPort   () << endl;
-	cout << "\tpath <"   << url.getPath   () << ">" << endl;
+	cout << "\tscheme <" << url.get_scheme () << ">" << endl;
+	cout << "\tuser <"   << url.get_user   () << ">" << endl;
 
-	vector<string> params = url.getQuery ();
+	Secret sec = url.get_secret ();
+	cout << "\tpasswd <" << sec.password () << ">" << endl;
+	sec.password_unref ();
+
+	cout << "\tauth <"   << url.get_auth   () << ">" << endl;
+	cout << "\thost <"   << url.get_host   () << ">" << endl;
+	cout << "\tport "    << url.get_port   () << endl;
+	cout << "\tpath <"   << url.get_path   () << ">" << endl;
+
+	vector<string> params = url.get_query ();
 	for (vector<string>::size_type i = 0; i != params.size (); i++) {
 	  cout << "\tquery[" << i << "] <"  << params[i] << ">" << endl;
 	}
       }
       catch (Exception& e) {
-	cerr << e.method () << ": " << e.msgError () << endl;
+	cerr << e.method () << ": " << e.what () << endl;
 	goto again;
       }
     }
