@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2007, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -13,9 +13,8 @@
    GNU Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General Public
-   License along with GNU Mailutils; if not, write to the Free
-   Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   License along with GNU Mailutils.  If not, see
+   <http://www.gnu.org/licenses/>. */; 
 
 /*  This is an example on how to write extension tests for GNU sieve.
     It provides test "numaddr".
@@ -40,7 +39,7 @@
 #endif  
 
 #include <stdlib.h>
-#include <mailutils/libsieve.h>
+#include <mailutils/sieve.h>
 
 struct val_ctr {  /* Data passed to the counter function */
   mu_header_t hdr;   /* Headers of the current message */
@@ -87,11 +86,7 @@ numaddr_test (mu_sieve_machine_t mach, mu_list_t args, mu_list_t tags)
   
   if (mu_sieve_get_debug_level (mach) & MU_SIEVE_DEBUG_TRACE)
     {
-      mu_sieve_locus_t locus;
-      mu_sieve_get_locus (mach, &locus);
-      mu_sieve_debug (mach, "%s:%lu: NUMADDR\n",
-		   locus.source_file,
-		   (unsigned long) locus.source_line);
+      mu_sieve_debug (mach, "NUMADDR");
     }
 
   /* Retrieve required arguments: */
