@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2005, 2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2005, 2007, 2008, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <frm.h>
 
@@ -22,8 +21,7 @@ int count_only;
 char *sender_option;
 char *mailbox_name;
 
-const char *program_version = "from (" PACKAGE_STRING ")";
-static char doc[] = N_("GNU from -- display from and subject");
+static char doc[] = N_("GNU from -- display from and subject.");
 
 static struct argp_option options[] = {
   {"count",  'c', NULL,   0, N_("just print a count of messages and exit")},
@@ -72,7 +70,6 @@ static struct argp argp = {
 static const char *capa[] = {
   "common",
   "debug",
-  "license",
   "mailbox",
   "locking",
   NULL
@@ -119,7 +116,7 @@ main (int argc, char **argv)
   mu_gocs_register ("tls", mu_tls_module_init);
 #endif
 
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   if (mu_app_init (&argp, capa, NULL, argc, argv, 0, &c, NULL))
     exit (1);
 
@@ -147,7 +144,7 @@ main (int argc, char **argv)
 
   if (count_only)
     {
-      printf (ngettext ("There is %lu message in your incoming mailbox.\n",
+      mu_printf (ngettext ("There is %lu message in your incoming mailbox.\n",
 			"There are %lu messages in your incoming mailbox.\n",
 			total),
 	      (unsigned long) total);
