@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2006, 2007, 2008, 2009, 2010 Free
-   Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2006, 2007, 2008, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "imap4d.h"
 
@@ -45,6 +43,7 @@ _append_sender (mu_envelope_t envelope, char *buf, size_t len, size_t *pnwrite)
   return 0;
 }
 
+/* FIXME: Why not use mu_message_size instead? */
 static int
 _append_size (mu_message_t msg, size_t *psize)
 {
@@ -66,42 +65,39 @@ imap4d_append0 (mu_mailbox_t mbox, int flags, char *date_time, char *text,
 {
   mu_stream_t stream;
   int rc = 0;
-  size_t len = 0;
   mu_message_t msg = 0;
   struct tm *tm;
   time_t t;
   mu_envelope_t env;
-    
+  size_t size;
+  
   if (mu_message_create (&msg, &tm))
     return 1;
-  
-  if (mu_memory_stream_create (&stream, 0, MU_STREAM_RDWR)
-      || mu_stream_open (stream))
-    {
-      mu_message_destroy (&msg, &tm);
-      return 1;
-    }
 
   /* If a date_time is specified, the internal date SHOULD be set in the
      resulting message; otherwise, the internal date of the resulting
      message is set to the current date and time by default. */
   if (date_time)
     {
-      if (util_parse_internal_date (date_time, &t))
+      if (util_parse_internal_date (date_time, &t, datetime_default))
 	{
 	  *err_text = "Invalid date/time format";
 	  return 1;
 	}
     }
   else
-    time(&t);
+    time (&t);
   
-  tm = gmtime(&t);
+  tm = gmtime (&t);
 
-  while (*text && mu_isblank (*text))
-    text++;
+  text = mu_str_skip_class (text, MU_CTYPE_BLANK);
+
+  if (mu_static_memory_stream_create (&stream, text, strlen (text)))
+    {
+      mu_message_destroy (&msg, &tm);
+      return 1;
+    }
 
-  mu_stream_write (stream, text, strlen (text), len, &len);
   mu_message_set_stream (msg, stream, &tm);
   mu_message_set_size (msg, _append_size, &tm);
 
@@ -109,17 +105,44 @@ imap4d_append0 (mu_mailbox_t mbox, int flags, char *date_time, char *text,
   mu_envelope_set_date (env, _append_date, msg);
   mu_envelope_set_sender (env, _append_sender, msg);
   mu_message_set_envelope (msg, env, &tm);
-  rc = mu_mailbox_append_message (mbox, msg);
-  if (rc == 0 && flags)
+
+  rc = _append_size (msg, &size);
+  if (rc)
     {
-      size_t num = 0;
-      mu_attribute_t attr = NULL;
-      mu_mailbox_messages_count (mbox, &num);
-      mu_mailbox_get_message (mbox, num, &msg);
-      mu_message_get_attribute (msg, &attr);
-      mu_attribute_set_flags (attr, flags);
+      mu_diag_output (MU_DIAG_NOTICE,
+		      _("cannot compute size of the message being appended; "
+			"using estimated value: %s"),
+		      mu_strerror (rc));
+      /* raw estimate */
+      size = strlen (text);
+    }
+  rc = quota_check (size);
+  if (rc != RESP_OK)
+    {
+      *err_text = rc == RESP_NO ?
+	                   "Mailbox quota exceeded" : "Operation failed";
+      mu_message_destroy (&msg, &tm);
+      return 1;
     }
 
+  imap4d_enter_critical ();
+  rc = mu_mailbox_append_message (mbox, msg);
+  if (rc == 0)
+    {
+      if (flags)
+	{
+	  size_t num = 0;
+	  mu_attribute_t attr = NULL;
+	  mu_mailbox_messages_count (mbox, &num);
+	  mu_mailbox_get_message (mbox, num, &msg);
+	  mu_message_get_attribute (msg, &attr);
+	  mu_attribute_set_flags (attr, flags);
+	}
+      /* FIXME: If not INBOX */
+      quota_update (size);
+    }
+  imap4d_leave_critical ();
+  
   mu_message_destroy (&msg, &tm);
   return rc;
 }
@@ -140,11 +163,11 @@ imap4d_append (struct imap4d_command *command, imap4d_tokbuf_t tok)
   char *err_text = "[TRYCREATE] failed";
   
   if (argc < 4)
-    return util_finish (command, RESP_BAD, "Too few arguments");
+    return io_completion_response (command, RESP_BAD, "Too few arguments");
       
   mboxname = imap4d_tokbuf_getarg (tok, IMAP4_ARG_1);
   if (!mboxname)
-    return util_finish (command, RESP_BAD, "Too few arguments");
+    return io_completion_response (command, RESP_BAD, "Too few arguments");
 
   i = IMAP4_ARG_2;
   if (imap4d_tokbuf_getarg (tok, i)[0] == '(')
@@ -160,7 +183,8 @@ imap4d_append (struct imap4d_command *command, imap4d_tokbuf_t tok)
 	    break;
 	}
       if (i == argc)
-	return util_finish (command, RESP_BAD, "Missing closing parenthesis");
+	return io_completion_response (command, RESP_BAD, 
+	                               "Missing closing parenthesis");
       i++;
     }
 
@@ -177,14 +201,14 @@ imap4d_append (struct imap4d_command *command, imap4d_tokbuf_t tok)
       break;
 
     default:
-      return util_finish (command, RESP_BAD, "Too many arguments");
+      return io_completion_response (command, RESP_BAD, "Too many arguments");
     }
 
   msg_text = imap4d_tokbuf_getarg (tok, i);
   
   mboxname = namespace_getfullpath (mboxname, "/", NULL);
   if (!mboxname)
-    return util_finish (command, RESP_NO, "Couldn't open mailbox"); 
+    return io_completion_response (command, RESP_NO, "Couldn't open mailbox"); 
 
   status = mu_mailbox_create_default (&dest_mbox, mboxname);
   if (status == 0)
@@ -202,9 +226,9 @@ imap4d_append (struct imap4d_command *command, imap4d_tokbuf_t tok)
   
   free (mboxname);
   if (status == 0)
-    return util_finish (command, RESP_OK, "Completed");
+    return io_completion_response (command, RESP_OK, "Completed");
 
-  return util_finish (command, RESP_NO, err_text);
+  return io_completion_response (command, RESP_NO, "%s", err_text);
 }
 
 
