@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2005, 2007, 2010, 2011 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_ERROR_H
 #define _MAILUTILS_ERROR_H
@@ -22,20 +22,15 @@
 #include <stdarg.h>
 
 #include <mailutils/types.h>
+#include <mailutils/diag.h>
 
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-typedef int (*mu_error_pfn_t) (const char *fmt, va_list ap);
-
 extern int mu_verror (const char *fmt, va_list ap);
 extern int mu_error (const char *fmt, ...) MU_PRINTFLIKE(1,2);
-extern void mu_error_set_print (mu_error_pfn_t);
 
-int mu_default_error_printer (const char *fmt, va_list ap);
-int mu_syslog_error_printer (const char *fmt, va_list ap);
-  
 #ifdef __cplusplus
 }
 #endif
