@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 2004, 2005, 2007, 2010, 2011 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,9 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MAILUTILS_NNTP_H
 #define _MAILUTILS_NNTP_H
@@ -44,8 +44,6 @@ extern int  mu_nntp_disconnect      (mu_nntp_t nntp);
 extern int  mu_nntp_set_timeout     (mu_nntp_t nntp, int timeout);
 extern int  mu_nntp_get_timeout     (mu_nntp_t nntp, int *timeout);
 
-extern int  mu_nntp_set_debug       (mu_nntp_t nntp, mu_debug_t debug);
-
 extern int  mu_nntp_stls            (mu_nntp_t nntp);
 
 
