@@ -1,6 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #include <mailutils/cpp/mailer.h>
 
@@ -80,15 +78,3 @@ Mailer :: send_message (const Message& msg, const Address& from,
     throw Exception ("Mailer::send_message", status);
 }
 
-Debug&
-Mailer :: get_debug ()
-{
-  mu_debug_t c_dbg;
-
-  int status = mu_mailer_get_debug (mailer, &c_dbg);
-  if (status)
-    throw Exception ("Mailer::get_debug", status);
-
-  return *new Debug (c_dbg);
-}
-
