@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2006, 2007, 
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2006, 2007, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,17 +13,19 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #include "mu_scm.h"
 #include <mailutils/io.h>
 
+#ifndef HAVE_SCM_T_OFF
+typedef off_t scm_t_off;
+#endif
+
 struct mu_port
 {
-  mu_stream_t stream;         /* Associated stream */
-  int offset;              /* Current offset in the stream */
+  mu_stream_t stream;      /* Associated stream */
   SCM msg;                 /* Message the port belongs to */		
 };
 
@@ -81,22 +83,20 @@ mu_port_make_from_stream (SCM msg, mu_stream_t stream, long mode)
   struct mu_port *mp;
   SCM port;
   scm_port *pt;
+  int flags;
   
   mp = scm_gc_malloc (sizeof (struct mu_port), "mu-port");
   mp->msg = msg;
   mp->stream = stream;
-  mp->offset = 0;
 
-  SCM_NEWCELL (port);
-  SCM_DEFER_INTS;
-  pt = scm_add_to_port_table (port);
-  SCM_SETPTAB_ENTRY (port, pt);
-  pt->rw_random = mu_stream_is_seekable (stream);
-  SCM_SET_CELL_TYPE (port, (scm_tc16_smuport | mode));
+  port = scm_new_port_table_entry (scm_tc16_smuport | mode);
+  pt = SCM_PTAB_ENTRY (port);
+  mu_stream_get_flags (stream, &flags);
+  pt->rw_random = flags & MU_STREAM_SEEK;
   SCM_SETSTREAM (port, mp);
   mu_port_alloc_buffer (port, 0, 0);
-  SCM_ALLOW_INTS;
-  /*  SCM_PTAB_ENTRY (port)->file_name = "name";FIXME*/
+  /* FIXME:
+     SCM_PTAB_ENTRY (port)->file_name = "name";*/
   return port;
 }
 
@@ -117,13 +117,13 @@ mu_port_flush (SCM port)
   struct mu_port *mp = MU_PORT (port);
   scm_port *pt = SCM_PTAB_ENTRY (port);
   int wrsize = pt->write_pos - pt->write_buf;
-  size_t n;
   
   if (wrsize)
     {
-      if (mu_stream_write (mp->stream, pt->write_buf, wrsize, mp->offset, &n))
-	return;
-      mp->offset += n;
+      int status = mu_stream_write (mp->stream, pt->write_buf, wrsize, NULL);
+      if (status)
+ 	mu_scm_error ("mu_port_flush", status,
+ 		      "Error writing to stream", SCM_BOOL_F);
     }
   pt->write_pos = pt->write_buf;
   pt->rw_active = SCM_PORT_NEITHER;
@@ -150,8 +150,10 @@ mu_port_close (SCM port)
 static scm_sizet
 mu_port_free (SCM port)
 {
+  struct mu_port *mp = MU_PORT (port);
+  mu_stream_unref (mp->stream);
   mu_port_close (port);
-  return sizeof (struct mu_port); /*FIXME: approximate */
+  return 0;
 }
 
 static int
@@ -162,8 +164,8 @@ mu_port_fill_input (SCM port)
   size_t nread = 0;
   int status;
   
-  status = mu_stream_read (mp->stream, pt->read_buf, pt->read_buf_size,
-			   mp->offset, &nread);
+  status = mu_stream_read (mp->stream, (char*) pt->read_buf, pt->read_buf_size,
+			   &nread);
   if (status)
     mu_scm_error ("mu_port_fill_input", status,
 		  "Error reading from stream", SCM_BOOL_F);
@@ -171,7 +173,6 @@ mu_port_fill_input (SCM port)
   if (nread == 0)
     return EOF;
 
-  mp->offset += nread;
   pt->read_pos = pt->read_buf;
   pt->read_end = pt->read_buf + nread;
   return *pt->read_buf;
@@ -213,21 +214,20 @@ mu_port_end_input (SCM port, int offset)
   if (offset > 0)
     {
       pt->read_pos = pt->read_end;
-      mp->offset -= delta;
+      mu_stream_seek (mp->stream, - delta, MU_SEEK_CUR, NULL);
     }
   pt->rw_active = SCM_PORT_NEITHER;
 }
 
-static mu_off_t
-mu_port_seek (SCM port, mu_off_t offset, int whence)
+static scm_t_off
+mu_port_seek (SCM port, scm_t_off offset, int whence)
 {
   struct mu_port *mp = MU_PORT (port);
   scm_port *pt = SCM_PTAB_ENTRY (port);
-  mu_off_t size = 0;
+  int mwhence;
+  mu_off_t pos;
+  int status;
   
-  if (whence == SEEK_CUR && offset == 0)
-    return mp->offset;
-
   if (pt->rw_active == SCM_PORT_WRITE)
     {
       mu_port_flush (port);
@@ -237,22 +237,22 @@ mu_port_seek (SCM port, mu_off_t offset, int whence)
       scm_end_input (port);
     }
 
-  mu_stream_size (mp->stream, &size);
   switch (whence)
     {
     case SEEK_SET:
+      mwhence = MU_SEEK_SET;
       break;
     case SEEK_CUR:
-      offset += mp->offset;
+      mwhence = MU_SEEK_CUR;
       break;
     case SEEK_END:
-      offset += size;
+      mwhence = MU_SEEK_END;
     }
 
-  if (offset > size)
-    return -1;
-  mp->offset = offset;
-  return offset;
+  status = mu_stream_seek (mp->stream, offset, mwhence, &pos);
+  if (status)
+    pos = -1;
+  return (scm_t_off) pos;
 }
 
 static void
