@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005,
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -51,7 +49,7 @@ mail_mbox_close ()
   
   mu_mailbox_get_url (mbox, &url);
   mu_mailbox_messages_count (mbox, &held_count);
-  fprintf (ofile, 
+  mu_printf (
            ngettext ("Held %d message in %s\n",
                      "Held %d messages in %s\n",
                      held_count),
@@ -118,14 +116,14 @@ mail_mbox_commit ()
 	       
 	      if ((status = mu_mailbox_create_default (&dest_mbox, name)) != 0)
 		{
-		  util_error (_("Cannot create mailbox %s: %s"), name,
+		  mu_error (_("Cannot create mailbox %s: %s"), name,
                               mu_strerror (status));
 		  return 1;
 		}
               if ((status = mu_mailbox_open (dest_mbox,
 			   	          MU_STREAM_WRITE | MU_STREAM_CREAT))!=0)
 		{
-		  util_error (_("Cannot open mailbox %s: %s"), name,
+		  mu_error (_("Cannot open mailbox %s: %s"), name,
                               mu_strerror (status));
 		  return 1;
 		}
@@ -133,7 +131,7 @@ mail_mbox_commit ()
 
 	  status = mu_mailbox_append_message (dest_mbox, msg);
 	  if (status)
-	    util_error (_("Cannot append message: %s"), mu_strerror (status));
+	    mu_error (_("Cannot append message: %s"), mu_strerror (status));
 	  else
 	    {
 	      mu_attribute_set_deleted (attr);
@@ -155,7 +153,7 @@ mail_mbox_commit ()
       mu_url_t u = NULL;
 
       mu_mailbox_get_url (dest_mbox, &u);
-      fprintf(ofile, 
+      mu_printf (
               ngettext ("Saved %d message in %s\n",
                         "Saved %d messages in %s\n",
 			saved_count),
