@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -23,16 +22,18 @@
  */
 
 static int
-size0 (msgset_t *mspec, message_t msg, void *data)
+size0 (msgset_t *mspec, mu_message_t msg, void *data)
 {
   size_t size = 0, lines = 0;
 
-  message_size (msg, &size);
-  message_lines (msg, &lines);
+  mu_message_size (msg, &size);
+  mu_message_lines (msg, &lines);
   
-  fprintf (ofile, "%c%2d %3d/%-5d\n",
-	   mspec->msg_part[0] == cursor ? '>' : ' ',
-	   mspec->msg_part[0], lines, size);
+  mu_printf ("%c%2lu %3lu/%-5lu\n",
+		    is_current_message (mspec->msg_part[0]) ? '>' : ' ',
+		    (unsigned long) mspec->msg_part[0],
+		    (unsigned long) lines,
+		    (unsigned long) size);
   return 0;
 }
 
