@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003,
-   2004, 2005, 2007, 2008 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2007, 2008,
+   2009, 2010, 2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #ifdef HAVE_CONFIG_H
 # include <config.h>
@@ -31,7 +29,6 @@
 
 static int messages_count (const char *);
 
-const char *program_version = "messages (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU messages -- count the number of messages in a mailbox");
 static char args_doc[] = N_("[mailbox...]");
 
@@ -39,15 +36,14 @@ static struct argp_option options[] = {
   { NULL,         0, NULL,  0,
     /* TRANSLATORS: 'messages' is a program name. Do not translate it! */
     N_("messages specific switches:"), 0},
-  {"quiet",	'q',	0,	0,	N_("Only display number of messages")},
-  {"silent",	's',	0,	0,	N_("Same as -q")},
+  {"quiet",	'q',	NULL,	0,	N_("only display number of messages")},
+  {"silent",	's',	NULL,	OPTION_ALIAS, NULL },
   { 0 }
 };
 
 static const char *argp_capa[] = {
   "common",
   "debug",
-  "license",
   "mailbox",
   "locking",
   NULL
@@ -112,7 +108,7 @@ main (int argc, char **argv)
 #ifdef WITH_TLS
   mu_gocs_register ("tls", mu_tls_module_init);
 #endif
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   if (mu_app_init (&argp, argp_capa, NULL, argc, argv, 0, NULL, &args))
     exit (1);
 
@@ -142,10 +138,10 @@ messages_count (const char *box)
   if (status != 0)
     {
       if (box)
-	mu_error (_("Could not create mailbox `%s': %s"),
+	mu_error (_("could not create mailbox `%s': %s"),
 		  box, mu_strerror (status));
       else
-	mu_error (_("Could not create default mailbox: %s"),
+	mu_error (_("could not create default mailbox: %s"),
 		  mu_strerror (status));
       return -1;
     }
@@ -156,7 +152,7 @@ messages_count (const char *box)
   status =  mu_mailbox_open (mbox, MU_STREAM_READ);
   if (status != 0)
     {
-      mu_error (_("Could not open mailbox `%s': %s"),
+      mu_error (_("could not open mailbox `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
@@ -164,20 +160,21 @@ messages_count (const char *box)
   status = mu_mailbox_messages_count (mbox, &count);
   if (status != 0)
     {
-      mu_error (_("Could not count messages in mailbox `%s': %s"),
+      mu_error (_("could not count messages in mailbox `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
 
   if (silent)
-    printf ("%d\n", count);
+    mu_printf ("%lu\n", (unsigned long) count);
   else
-    printf (_("Number of messages in %s: %d\n"), box, count);
+    mu_printf (_("Number of messages in %s: %lu\n"), box,
+	       (unsigned long) count);
 
   status = mu_mailbox_close (mbox);
   if (status != 0)
     {
-      mu_error (_("Could not close `%s': %s"),
+      mu_error (_("could not close `%s': %s"),
 		box, mu_strerror (status));
       return -1;
     }
