@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2006, 2007, 2009, 2010
-   Free Software Foundation, Inc.
+   Copyright (C) 1999, 2000, 2001, 2002, 2005, 2006, 2007, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Coexistence between GNU long options, traditional UNIX-style short
    options and traditional MH long options. */
@@ -28,6 +26,9 @@
 #include <string.h>
 #include <mailutils/argcv.h>
 #include "argp.h"
+#ifdef MU_ALPHA_RELEASE
+# include <git-describe.h>
+#endif
 
 static error_t
 parse_opt (int key, char *arg, struct argp_state *state)
@@ -93,11 +94,49 @@ my_argp_parse (struct argp *argp, int argc, char **argv, int flags,
   return rc;
 }
 
+const char version_etc_copyright[] =
+  /* Do *not* mark this string for translation.  %s is a copyright
+     symbol suitable for this locale, and %d is the copyright
+     year.  */
+  "Copyright %s 2010 Free Software Foundation, inc.";
+
+/* This is almost the same as mu_program_version_hook from muinit.c,
+   except for different formatting of the first line. MH uses:
+
+      progname (GNU Mailutils X.Y.Z)
+
+   where X.Y.Z stands for the version number. Emacs MH-E uses this
+   to determine Mailutils presence and its version number (see
+   lisp/mh-e/mh-e.el, function mh-variant-mu-mh-info). */
+static void
+mh_program_version_hook (FILE *stream, struct argp_state *state)
+{
+#ifdef GIT_DESCRIBE
+  fprintf (stream, "%s (%s %s) [%s]\n",
+	   mu_program_name, PACKAGE_NAME, PACKAGE_VERSION, GIT_DESCRIBE);
+#else
+  fprintf (stream, "%s (%s %s)\n", mu_program_name,
+	   PACKAGE_NAME, PACKAGE_VERSION);
+#endif
+  /* TRANSLATORS: Translate "(C)" to the copyright symbol
+     (C-in-a-circle), if this symbol is available in the user's
+     locale.  Otherwise, do not translate "(C)"; leave it as-is.  */
+  fprintf (stream, version_etc_copyright, _("(C)"));
+
+  fputs (_("\
+\n\
+License GPLv3+: GNU GPL version 3 or later <http://gnu.org/licenses/gpl.html>\nThis is free software: you are free to change and redistribute it.\n\
+There is NO WARRANTY, to the extent permitted by law.\n\
+\n\
+"),
+	 stream);
+}
+
 void
-mh_argp_init (const char *vers)
+mh_argp_init ()
 {
-  argp_program_version = vers ? vers : PACKAGE_STRING;
   argp_program_bug_address =  "<" PACKAGE_BUGREPORT ">";
+  argp_program_version_hook = mh_program_version_hook;
 }
 
 
@@ -122,7 +161,7 @@ mu_debug_argp_parser (int key, char *arg, struct argp_state *state)
   switch (key)
     {
     case OPT_DEBUG_LEVEL:
-      mu_global_debug_from_string (arg, "command line");
+      mu_debug_parse_spec (arg);
       break;
 
     case OPT_DEBUG_LINE_INFO:
@@ -141,7 +180,7 @@ struct argp mu_debug_argp = {
 };
 
 struct argp_child mh_argp_children[] = {
-  { &mu_debug_argp, 0, N_("Global debugging settings"), 0 },
+  { &mu_debug_argp, 0, N_("Global debugging settings"), -2 },
   { NULL }
 };
 
@@ -162,7 +201,8 @@ mh_argp_parse (int *pargc, char **pargv[],
 
   mu_set_program_name ((*pargv)[0]);
   mh_init ();
-  
+
+  mh_option_init (mh_option);
   memset (&argp, 0, sizeof (argp));
   argp.options = option;
   argp.parser = parse_opt;
@@ -180,24 +220,31 @@ mh_argp_parse (int *pargc, char **pargv[],
     {
       int argc;
       char **argv;
-      int xargc;
-      char **xargv;
       int i, j;
+      struct mu_wordsplit ws;
       
-      mu_argcv_get (val, "", NULL, &xargc, &xargv);
+      if (mu_wordsplit (val, &ws, MU_WRDSF_DEFFLAGS))
+	{
+	  mu_error (_("cannot split line `%s': %s"), val,
+		    mu_wordsplit_strerror (&ws));
+	  exit (1);
+	}
 
-      argc = *pargc + xargc;
-      argv = calloc (argc+1, sizeof *argv);
+      argc = *pargc + ws.ws_wordc;
+      argv = calloc (argc + 1, sizeof *argv);
       if (!argv)
         mh_err_memory (1);
 
       i = 0;
       argv[i++] = (*pargv)[0];
-      for (j = 0; j < xargc; i++, j++)
-	argv[i] = xargv[j];
+      for (j = 0; j < ws.ws_wordc; i++, j++)
+	argv[i] = ws.ws_wordv[j];
       for (j = 1; i < argc; i++, j++)
 	argv[i] = (*pargv)[j];
       argv[i] = NULL;
+
+      ws.ws_wordc = 0;
+      mu_wordsplit_free (&ws);
       
       mh_argv_preproc (argc, argv, &data);
 
@@ -207,7 +254,6 @@ mh_argp_parse (int *pargc, char **pargv[],
 
       *pargc = argc;
       *pargv = argv;
-      free (xargv);
     }
   else
     {
@@ -242,8 +288,7 @@ mh_license (const char *name)
     "   GNU General Public License for more details.\n"
     "\n"
     "   You should have received a copy of the GNU General Public License along\n"
-    "   with GNU Mailutils; if not, write to the Free Software Foundation,\n"
-    "   Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA\n"
+    "   with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.\n"
     "\n"
     "\n"
 ));
