@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 
-   2007, 2008, 2009 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
+   2011 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH refile command */
 
@@ -26,11 +24,10 @@
 #include <errno.h>
 #include <fcntl.h>
 
-const char *program_version = "refile (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH refile")"\v"
 N_("Options marked with `*' are not yet implemented.\n\
 Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("messages folder [folder...]");
+static char args_doc[] = N_("MSGLIST FOLDER [FOLDER...]");
 
 /* GNU options */
 static struct argp_option options[] = {
@@ -47,19 +44,17 @@ static struct argp_option options[] = {
    N_("specify source folder; it will become the current folder after the program exits")},
   {"src", 0, NULL, OPTION_ALIAS, NULL},
   {"file", ARG_FILE, N_("FILE"), 0, N_("use FILE as the source message")},
-  {"license", ARG_LICENSE, 0,      0,
-   N_("display software license"), -1},
   { 0 }
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"file",     2, 0, "input-file"},
-  {"draft",    1, 0, NULL },
-  {"link",     1, MH_OPT_BOOL, NULL },
-  {"preserve", 1, MH_OPT_BOOL, NULL },
-  {"src",      1, 0, "+folder" },
-  { 0 }
+  { "file",     MH_OPT_ARG, "input-file"},
+  { "draft" },
+  { "link",     MH_OPT_BOOL },
+  { "preserve", MH_OPT_BOOL },
+  { "src",      MH_OPT_ARG, "folder" },
+  { NULL }
 };
 
 int link_flag = 0;
@@ -109,7 +104,7 @@ open_folders ()
       mu_mailbox_t mbox;
       
       mu_iterator_current (itr, (void **)&name);
-      mbox = mh_open_folder (name, 1);
+      mbox = mh_open_folder (name, MU_STREAM_RDWR|MU_STREAM_CREAT);
       mu_list_append (folder_mbox_list, mbox);
       free (name);
     }
@@ -151,7 +146,7 @@ close_folders ()
 }
 
 static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -168,6 +163,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
       
     case ARG_PRESERVE:
+      mh_opt_notimpl_warning ("-preserve");
       preserve_flag = is_true(arg);
       break;
 	
@@ -179,12 +175,8 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       source_file = arg;
       break;
       
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -232,7 +224,7 @@ main (int argc, char **argv)
   /* Native Language Support */
   MU_APP_INIT_NLS ();
 
-  mh_argp_init (program_version);
+  mh_argp_init ();
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
@@ -269,7 +261,7 @@ main (int argc, char **argv)
     }
   else
     {
-      mbox = mh_open_folder (mh_current_folder (), 0);
+      mbox = mh_open_folder (mh_current_folder (), MU_STREAM_RDWR);
       mh_msgset_parse (mbox, &msgset, argc, argv, "cur");
 
       status = mh_iterate (mbox, &msgset, refile_iterator, NULL);
