@@ -1,9 +1,10 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2005, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
+   the Free Software Foundation; either version 3, or (at your option)
    any later version.
 
    GNU Mailutils is distributed in the hope that it will be useful,
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH rmf command */
 
@@ -29,44 +28,39 @@
 
 #include <dirent.h>
 
-const char *program_version = "rmf (" PACKAGE_STRING ")";
-/* TRANSLATORS: Please, preserve the vertical tabulation (^K character)
-   in this message */
-static char doc[] = N_("GNU MH rmf\v\
-Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("[+folder]");
+static char doc[] = N_("GNU MH rmf")"\v"
+N_("Use -help to obtain the list of traditional MH options.");
+static char args_doc[] = N_("[+FOLDER]");
 
 /* GNU options */
 static struct argp_option options[] = {
   {"folder",  ARG_FOLDER, N_("FOLDER"), 0,
-   N_("Specify the folder to delete")},
+   N_("specify the folder to delete")},
   {"interactive", ARG_INTERACTIVE, N_("BOOL"), OPTION_ARG_OPTIONAL,
-    N_("Interactive mode: ask for confirmation before removing each folder")},
+    N_("interactive mode: ask for confirmation before removing each folder")},
   {"nointeractive", ARG_NOINTERACTIVE, NULL, OPTION_HIDDEN, ""},
   {"recursive", ARG_RECURSIVE, NULL, 0,
-   N_("Recursively delete all subfolders")},
+   N_("recursively delete all subfolders")},
   {"norecursive", ARG_NORECURSIVE, NULL, OPTION_HIDDEN, ""},
-  {"license", ARG_LICENSE, 0,      0,
-   N_("Display software license"), -1},
   { 0 }
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"interactive", 1, MH_OPT_BOOL, NULL},
+  { "interactive", MH_OPT_BOOL },
   { 0 }
 };
 
 int explicit_folder; /* Was the folder explicitly given */
 int interactive; /* Ask for confirmation before deleting */
-int recurse;     /* Recursively process all the sub-directories */
+int recursive;     /* Recursively process all the sub-directories */
 
 static char *cur_folder_path; /* Full pathname of the current folder */
 static char *folder_name;     /* Name of the (topmost) folder to be
 				 deleted */
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -84,19 +78,15 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       break;
 	
     case ARG_RECURSIVE:
-      recurse = is_true (arg);
+      recursive = is_true (arg);
       break;
       
     case ARG_NORECURSIVE:
-      recurse = 0;
-      break;
-
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
+      recursive = 0;
       break;
 
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
@@ -104,7 +94,7 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
 static char *
 current_folder_path ()
 {
-  mu_mailbox_t mbox = mh_open_folder (mh_current_folder (), 0);
+  mu_mailbox_t mbox = mh_open_folder (mh_current_folder (), MU_STREAM_RDWR);
   mu_url_t url;
   char *p;
   mu_mailbox_get_url (mbox, &url);
@@ -116,6 +106,31 @@ current_folder_path ()
 
 static int
 rmf (const char *name)
+{
+  mu_mailbox_t mbox = NULL;
+  int rc;
+  
+  rc = mu_mailbox_create_default (&mbox, name);
+  if (rc)
+    {
+      mu_error (_("cannot create mailbox %s: %s"),
+		name, strerror (rc));
+      return 1;
+    }
+
+  rc = mu_mailbox_remove (mbox);
+  mu_mailbox_destroy (&mbox);
+  if (rc)
+    {
+      mu_error (_("cannot remove %s: %s"), name, mu_strerror (rc));
+      return 1;
+    }
+  return 0;
+}
+
+/* Recursive rmf */
+static int
+recrmf (const char *name)
 {
   DIR *dir;
   struct dirent *entry;
@@ -125,7 +140,7 @@ rmf (const char *name)
 
   if (!dir)
     {
-      mu_error (_("Cannot scan folder %s: %s"), name, strerror (errno));
+      mu_error (_("cannot scan folder %s: %s"), name, strerror (errno));
       return 1;
     }
 
@@ -141,62 +156,36 @@ rmf (const char *name)
 	  || strcmp (entry->d_name, "..") == 0)
 	continue;
       
-      asprintf (&p, "%s/%s", name, entry->d_name);
+      p = mh_safe_make_file_name (name, entry->d_name);
       if (stat (p, &st) < 0)
 	{
-	  mu_error (_("Cannot stat %s: %s"), p, strerror (errno));
+	  mu_diag_funcall (MU_DIAG_ERROR, "stat", p, errno);
 	}
       else if (S_ISDIR (st.st_mode))
-	{
-	  if (recurse)
-	    failures += rmf (p);
-	  else
-	    {
-	      printf ("%s: file `%s' not deleted, continuing...\n",
-		      program_invocation_short_name, p);
-	      failures++;
-	    }
-	}
-      else
-	{
-	  if (unlink (p))
-	    {
-	      mu_error (_("Cannot unlink %s: %s"), p, strerror (errno));
-	      failures++;
-	    }
-	}
+	failures += recrmf (p);
       free (p);
     }
   closedir (dir);
 
   if (failures == 0)
-    failures += rmdir (name);
+    failures += rmf (name);
   else
     printf ("%s: folder `%s' not removed\n",
-	    program_invocation_short_name, name);
+	    mu_program_name, name);
 
-  if (failures == 0)
-    {
-      if (cur_folder_path && strcmp (name, cur_folder_path) == 0)
-	{
-	  current_folder = "inbox";
-	  mh_global_sequences_drop ();
-	  mh_global_save_state ();
-	  printf ("[+inbox now current]\n");
-	}
-    }
   return failures;
 }
 
 int
 main (int argc, char **argv)
 {
+  int status;
   char *name;
 
   /* Native Language Support */
-  mu_init_nls ();
+  MU_APP_INIT_NLS ();
 
-  mu_argp_init (program_version, NULL);
+  mh_argp_init ();
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, NULL);
 
@@ -209,6 +198,23 @@ main (int argc, char **argv)
     }
   else
     name = mh_expand_name (NULL, folder_name, 0);
-  rmf (name);
-  return 0;
+  if (recursive)
+    status = recrmf (name);
+  else
+    {
+      if (interactive && !mh_getyn (_("Remove folder %s"), name))
+	exit (0);
+      status = rmf (name);
+    }
+  if (status == 0)
+    {
+      if (cur_folder_path && strcmp (name, cur_folder_path) == 0)
+	{
+	  mh_set_current_folder ("inbox");
+	  mh_global_save_state ();
+	  printf ("[+inbox now current]\n");
+	}
+      return 0;
+    }      
+  return 1;
 }
