@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008, 
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* MH sortm command */
 
@@ -24,10 +22,9 @@
 #include <unistd.h>
 #include <signal.h>
 
-const char *program_version = "sortm (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH sortm")"\v"
 N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = N_("[msgs]");
+static char args_doc[] = N_("[MSGLIST]");
 
 #define ARG_QUICKSORT 1024
 #define ARG_SHELL     1025
@@ -76,21 +73,18 @@ static struct argp_option options[] = {
   {"quicksort", ARG_QUICKSORT,  0, 0,
    N_("use quicksort algorithm (default)"), 40 },
 
-  {"license", ARG_LICENSE, 0,      0,
-   N_("display software license"), -1},
-
   { NULL },
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"datefield",     1, 0, "field" },
-  {"nodatefield",   3, 0, 0 },
-  {"textfield",     1, 0, "field" },
-  {"notextfield",   3, 0, 0 },
-  {"limit",         1, 0, "days" },
-  {"nolimit",       3, 0, 0 },
-  {"verbose",       1, MH_OPT_BOOL, NULL},
+  { "datefield",     MH_OPT_ARG, "field" },
+  { "nodatefield" },
+  { "textfield",     MH_OPT_ARG, "field" },
+  { "notextfield" },
+  { "limit",         MH_OPT_ARG, "days" },
+  { "nolimit" },
+  { "verbose",       MH_OPT_BOOL },
   { NULL },
 };
 
@@ -99,6 +93,7 @@ static int verbose;
 static mu_mailbox_t mbox;
 static const char *mbox_path;
 static mh_msgset_t msgset;
+static size_t current_num;
 
 #define ACTION_REORDER   0
 #define ACTION_DRY_RUN   1
@@ -116,8 +111,8 @@ static int comp_text (void *a, void *b);
 static int comp_date (void *a, void *b);
 static int comp_number (void *a, void *b);
 
-static int
-opt_handler (int key, char *arg, void *unused, struct argp_state *state)
+static error_t
+opt_handler (int key, char *arg, struct argp_state *state)
 {
   switch (key)
     {
@@ -191,19 +186,16 @@ opt_handler (int key, char *arg, void *unused, struct argp_state *state)
       algorithm = key;
       break;
       
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
-      return 1;
+      return ARGP_ERR_UNKNOWN;
     }
   return 0;
 }
 
 
 /* *********************** Comparison functions **************************** */
-struct comp_op {
+struct comp_op
+{
   char *field;
   compfun comp;
 };
@@ -215,17 +207,22 @@ addop (char *field, compfun comp)
 {
   struct comp_op *op = xmalloc (sizeof (*op));
   
-  if (!oplist && mu_list_create (&oplist))
+  if (!oplist)
     {
-      mu_error (_("can't create operation list"));
-      exit (1);
+      if (mu_list_create (&oplist))
+	{
+	  mu_error (_("can't create operation list"));
+	  exit (1);
+	}
+      mu_list_set_destroy_item (oplist, mu_list_free_item);
     }
   op->field = field;
   op->comp = comp;
   mu_list_append (oplist, op);
 }
 
-struct rem_data {
+struct rem_data
+{
   struct comp_op *op;
   compfun comp;
 };
@@ -248,10 +245,10 @@ remop (compfun comp)
   d.op = NULL;
   mu_list_do (oplist, rem_action, &d);
   mu_list_remove (oplist, d.op);
-  free (d.op);
 }
 
-struct comp_data {
+struct comp_data
+{
   int r;
   mu_message_t m[2];
 };
@@ -454,8 +451,8 @@ swap_message (size_t a, size_t b)
   char *path_a, *path_b;
   char *tmp;
   
-  asprintf (&path_a, "%s/%s", mbox_path, mu_umaxtostr (0, a));
-  asprintf (&path_b, "%s/%s", mbox_path, mu_umaxtostr (1, b));
+  path_a = mh_safe_make_file_name (mbox_path, mu_umaxtostr (0, a));
+  path_b = mh_safe_make_file_name (mbox_path, mu_umaxtostr (1, b));
   tmp = mu_tempname (mbox_path);
   rename (path_a, tmp);
   unlink (path_a);
@@ -498,7 +495,7 @@ sort ()
   switch (algorithm)
     {
     case ARG_QUICKSORT:
-      qsort(msgset.list, msgset.count, sizeof(msgset.list[0]),
+      qsort(msgset.list, msgset.count, sizeof (msgset.list[0]),
 	    comp);
       break;
 
@@ -535,14 +532,36 @@ sort ()
 	      mh_message_number (msg, &new_num);
 	      transpose (i, oldlist[i]);
 	      if (verbose)
-		fprintf (stderr, "{%s, %s}\n",
-			 mu_umaxtostr (0, old_num),
-			 mu_umaxtostr (1, new_num));
+		{
+		  fprintf (stderr, "{%s, %s}",
+			   mu_umaxtostr (0, old_num),
+			   mu_umaxtostr (1, new_num));
+		}
+	      if (old_num == current_num)
+		{
+		  if (verbose)
+		    fputc ('*', stderr);
+		  current_num = new_num;
+		}
+	      else if (new_num == current_num)
+		{
+		  if (verbose)
+		    fputc ('*', stderr);
+		  current_num = old_num;
+		}
+	      if (verbose)
+		fputc ('\n', stderr);
 	      if (action == ACTION_REORDER)
 		swap_message (old_num, new_num);
 	    }
 	}
     }
+  if (action == ACTION_REORDER)
+    {
+      mu_mailbox_close (mbox);
+      mu_mailbox_open (mbox, MU_STREAM_RDWR);
+      mh_mailbox_set_cur (mbox, current_num);
+    }
 }
 
 
@@ -555,7 +574,7 @@ main (int argc, char **argv)
   mu_url_t url;
   
   MU_APP_INIT_NLS ();
-  mh_argp_init (program_version);
+  mh_argp_init ();
   mh_argp_parse (&argc, &argv, 0, options, mh_option,
 		 args_doc, doc, opt_handler, NULL, &index);
   if (!oplist)
@@ -567,7 +586,7 @@ main (int argc, char **argv)
       exit (1);
     }
   
-  mbox = mh_open_folder (mh_current_folder (), 0);
+  mbox = mh_open_folder (mh_current_folder (), MU_STREAM_READ);
   mu_mailbox_get_url (mbox, &url);
   mbox_path = mu_url_to_string (url);
   if (memcmp (mbox_path, "mh:", 3) == 0)
@@ -576,7 +595,12 @@ main (int argc, char **argv)
   argc -= index;
   argv += index;
 
+  mh_mailbox_get_cur (mbox, &current_num);
+
   mh_msgset_parse (mbox, &msgset, argc, argv, "all");
-  sort (mbox, msgset);
+  sort ();
+  mh_global_save_state ();
+  mu_mailbox_destroy (&mbox);
+  
   return 0;
 }
