@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2007, 2008, 2009, 2010 Free Software Foundation,
-   Inc.
+   Copyright (C) 2003, 2007, 2008, 2009, 2010, 2011 Free Software
+   Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,16 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include <mh.h>
 
-const char *program_version = "whom (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU MH whom")"\v"
 N_("Use -help to obtain the list of traditional MH options.");
-static char args_doc[] = "[file]";
+static char args_doc[] = "[FILE]";
 
 /* GNU options */
 static struct argp_option options[] = {
@@ -40,20 +37,17 @@ static struct argp_option options[] = {
    N_("check if addresses are deliverable") },
   {"nocheck",       ARG_NOCHECK,       NULL, OPTION_HIDDEN, "" },
 
-  {"license",       ARG_LICENSE,       0,    0,
-   N_("display software license"), -1},
-
   {NULL}
 };
 
 /* Traditional MH options */
 struct mh_option mh_option[] = {
-  {"alias",         1, 0, "aliasfile" },
-  {"draft",         5, 0, NULL },
-  {"draftfolder",   6, 0, "folder" },
-  {"draftmessage",  6, 0, "message"},
-  {"nodraftfolder", 3, 0, NULL },
-  {"check",         1, MH_OPT_BOOL, NULL},
+  { "alias",         MH_OPT_ARG, "aliasfile" },
+  { "draft" },
+  { "draftfolder",   MH_OPT_ARG, "folder" },
+  { "draftmessage",  MH_OPT_ARG, "message" },
+  { "nodraftfolder" },
+  { "check",         MH_OPT_BOOL },
   {NULL}
 };
 
@@ -96,10 +90,6 @@ opt_handler (int key, char *arg, struct argp_state *state)
       check_recipients = 0;
       break;
 
-    case ARG_LICENSE:
-      mh_license (argp_program_version);
-      break;
-
     default:
       return ARGP_ERR_UNKNOWN;
     }
@@ -114,7 +104,7 @@ main (int argc, char **argv)
   
   MU_APP_INIT_NLS ();
   
-  mh_argp_init (program_version);
+  mh_argp_init ();
   mh_argp_parse (&argc, &argv, 0, options, mh_option, args_doc, doc,
 		 opt_handler, NULL, &index);
 
