@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 2003, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>.  */
 
 #if defined(HAVE_CONFIG_H)
 # include <config.h>
@@ -29,18 +27,18 @@
 #include <unistd.h>
 #include <mailutils/mailutils.h>
 #include <mailutils/tls.h>
-#include <mu_asprintf.h>
 #include "mailutils/libargp.h"
 #include <muaux.h>
 
-const char *program_version = "movemail (" PACKAGE_STRING ")";
 static char doc[] = N_("GNU movemail -- move messages across mailboxes.");
 static char args_doc[] = N_("inbox-url destfile [POP-password]");
 
 enum {
   EMACS_OPTION=256,
   IGNORE_ERRORS_OPTION,
-  PROGRAM_ID_OPTION
+  PROGRAM_ID_OPTION,
+  MAX_MESSAGES_OPTION,
+  ONERROR_OPTION
 };
 
 static struct argp_option options[] = {
@@ -57,8 +55,12 @@ static struct argp_option options[] = {
     N_("control mailbox ownership") },
   { "ignore-errors", IGNORE_ERRORS_OPTION, NULL, 0,
     N_("try to continue after errors") },
+  { "onerror", ONERROR_OPTION, N_("KW[,KW...]"), 0,
+    N_("what to do on errors") },
   { "program-id", PROGRAM_ID_OPTION, N_("FMT"), 0,
     N_("set program identifier for diagnostics (default: program name)") },
+  { "max-messages", MAX_MESSAGES_OPTION, N_("NUMBER"), 0,
+    N_("process at most NUMBER messages") },
   { NULL,      0, NULL, 0, NULL, 0 }
 };
 
@@ -69,6 +71,18 @@ static int uidl_option;
 static int verbose_option;
 static int ignore_errors;
 static char *program_id_option;
+static size_t max_messages_option;
+
+  /* These bits tell what to do when an error occurs: */
+#define ONERROR_SKIP     0x01  /* Skip to the next message */
+#define ONERROR_DELETE   0x02  /* Delete the source message */
+#define ONERROR_COUNT    0x04  /* Count it as processed */ 
+
+static int onerror_flags;  
+
+size_t msg_count = 0;         /* Number of processed messages */
+size_t get_err_count = 0;     /* Number of message retrieval errors */ 
+size_t app_err_count = 0;     /* Number of message appending errors */
 
 enum set_ownership_mode
   {
@@ -153,6 +167,14 @@ parse_opt (int key, char *arg, struct argp_state *state)
       mu_argp_node_list_new (lst, "ignore-errors", "yes");
       break;
 
+    case ONERROR_OPTION:
+      mu_argp_node_list_new (lst, "onerror", arg);
+      break;
+      
+    case MAX_MESSAGES_OPTION:
+      mu_argp_node_list_new (lst, "max-messages", arg);
+      break;
+      
     case PROGRAM_ID_OPTION:
       mu_argp_node_list_new (lst, "program-id", arg);
       break;
@@ -182,7 +204,7 @@ static struct argp argp = {
 
 
 static int
-_cb_mailbox_ownership (mu_debug_t debug, const char *str)
+_cb_mailbox_ownership (const char *str)
 {
   if (strcmp (str, "clear") == 0)
     so_method_num = 0;
@@ -195,9 +217,7 @@ _cb_mailbox_ownership (mu_debug_t debug, const char *str)
 	  
       if (mu_kwd_xlat_name_len (method_kwd, str, len, &code))
 	{
-	  mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-			       _("invalid ownership method: %s"),
-			       str);
+	  mu_error (_("invalid ownership method: %s"), str);
 	  return 1;
 	}
       
@@ -212,9 +232,7 @@ _cb_mailbox_ownership (mu_debug_t debug, const char *str)
 	case set_owner_id:
 	  if (!str[len])
 	    {
-	      mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-				   _("ownership method %s requires value"),
-				   str);
+	      mu_error (_("ownership method %s requires value"), str);
 	      return 1;
 	    }
 	  str += len + 1;
@@ -227,17 +245,13 @@ _cb_mailbox_ownership (mu_debug_t debug, const char *str)
 		  meth->owner.id.gid = strtoul (str, &p, 0);
 		  if (*p)
 		    {
-		      mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-					   _("expected gid number, but found %s"),
-					   str);
+		      mu_error (_("expected gid number, but found %s"), str);
 		      return 1;
 		    }
 		}
 	      else
 		{
-		  mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-				       _("expected uid number, but found %s"),
-				       str);
+		  mu_error (_("expected uid number, but found %s"), str);
 		  return 1;
 		}
 	    }
@@ -248,9 +262,7 @@ _cb_mailbox_ownership (mu_debug_t debug, const char *str)
 	case set_owner_name:
 	  if (!str[len])
 	    {
-	      mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-				   _("ownership method %s requires value"),
-				   str);
+	      mu_error (_("ownership method %s requires value"), str);
 	      return 1;
 	    }
 	  meth->owner.name = mu_strdup (str + len + 1);
@@ -260,7 +272,7 @@ _cb_mailbox_ownership (mu_debug_t debug, const char *str)
 }
 
 static int
-cb_mailbox_ownership (mu_debug_t debug, void *data, mu_config_value_t *val)
+cb_mailbox_ownership (void *data, mu_config_value_t *val)
 {
   int i;
   
@@ -268,43 +280,88 @@ cb_mailbox_ownership (mu_debug_t debug, void *data, mu_config_value_t *val)
     {
       const char *str = val->v.string;
       if (!strchr (str, ','))
-	return _cb_mailbox_ownership (debug, str);
+	return _cb_mailbox_ownership (str);
       else
 	{
-	  int argc;
-	  char **argv;
+	  struct mu_wordsplit ws;
 
-	  if (mu_argcv_get_np (str, strlen (str), ",", NULL, 0,
-			       &argc, &argv, NULL))
+	  ws.ws_delim = ",";
+	  if (mu_wordsplit (str, &ws, MU_WRDSF_DEFFLAGS|MU_WRDSF_DELIM))
 	    {
-	      mu_cfg_format_error (debug, MU_DEBUG_ERROR, 
-				   _("cannot parse %s"),
-				   str);
+	      mu_error (_("cannot parse %s: %s"),
+			str, mu_wordsplit_strerror (&ws));
 	      return 1;
 	    }
 
-	  for (i = 0; i < argc; i++)
-	    if (_cb_mailbox_ownership (debug, argv[i]))
+	  for (i = 0; i < ws.ws_wordc; i++)
+	    if (_cb_mailbox_ownership (ws.ws_wordv[i]))
 	      return 1;
-
-	  mu_argcv_free (argc, argv);
+	  mu_wordsplit_free (&ws);
 	  return 0;
 	}
     }
 		
-  if (mu_cfg_assert_value_type (val, MU_CFG_LIST, debug))
+  if (mu_cfg_assert_value_type (val, MU_CFG_LIST))
     return 1;
 
   for (i = 0; i < val->v.arg.c; i++)
     {
-      if (mu_cfg_assert_value_type (&val->v.arg.v[i], MU_CFG_STRING, debug))
+      if (mu_cfg_assert_value_type (&val->v.arg.v[i], MU_CFG_STRING))
 	return 1;
-      if (_cb_mailbox_ownership (debug, val->v.arg.v[i].v.string))
+      if (_cb_mailbox_ownership (val->v.arg.v[i].v.string))
 	return 1;
     }
   return 0;
 }
 
+static int
+cb_onerror (void *data, mu_config_value_t *val)
+{
+  struct mu_wordsplit ws;
+  static struct mu_kwd onerror_kw[] = {
+    { "skip", ONERROR_SKIP },
+    { "delete", ONERROR_DELETE },
+    { "count", ONERROR_COUNT },
+    { NULL }
+  };
+  int i, flag;
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  if (strcmp (val->v.string, "abort") == 0)
+    {
+      onerror_flags = 0;
+      return 0;
+    }
+  ws.ws_delim = ",";
+  if (mu_wordsplit (val->v.string, &ws,
+		    MU_WRDSF_NOVAR | MU_WRDSF_NOCMD |
+		    MU_WRDSF_DELIM | MU_WRDSF_WS))
+    {
+      mu_error (_("cannot split argument: %s"), mu_wordsplit_strerror (&ws));
+      return 1;
+    }
+  for (i = 0; i < ws.ws_wordc; i++)
+    {
+      int clr = 0;
+      char *name = ws.ws_wordv[i];
+      
+      if (strncmp (name, "no", 2) == 0)
+	{
+	  clr = 1;
+	  name += 2;
+	}
+      if (mu_kwd_xlat_name (onerror_kw, name, &flag))
+	mu_error (_("unknown keyword: %s"), ws.ws_wordv[i]);
+      if (clr)
+	onerror_flags &= ~flag;
+      else
+	onerror_flags |= flag;
+    }
+  mu_wordsplit_free (&ws);
+  return 0;
+}
+  
 struct mu_cfg_param movemail_cfg_param[] = {
   { "preserve", mu_cfg_bool, &preserve_mail, 0, NULL,
     N_("Do not remove messages from the source mailbox.") },
@@ -316,8 +373,6 @@ struct mu_cfg_param movemail_cfg_param[] = {
     N_("Use UIDLs to avoid downloading the same message twice.") },
   { "verbose", mu_cfg_int, &verbose_option, 0, NULL,
     N_("Set verbosity level.") },
-  { "ignore-errors", mu_cfg_bool, &ignore_errors, 0, NULL,
-    N_("Continue after an error.") },
   { "program-id", mu_cfg_string, &program_id_option, 0, NULL,
     N_("Set program identifier string (default: program name)") },
   { "mailbox-ownership", mu_cfg_callback, NULL, 0,
@@ -329,6 +384,18 @@ struct mu_cfg_param movemail_cfg_param[] = {
        " set-id=UID[:GID] set supplied UID and GID\n"
        " set-name=USER    make destination mailbox owned by USER"),
     N_("methods: list") },
+  { "max-messages", mu_cfg_size, &max_messages_option, 0, NULL,
+    N_("Copy at most <count> messages."),
+    N_("count") },
+  { "ignore-errors", mu_cfg_bool, &ignore_errors, 0, NULL,
+    N_("Continue after an error.") },
+  { "onerror", mu_cfg_callback, NULL, 0, cb_onerror,
+    N_("What to do after an error. Argument is a comma-separated list of:\n"
+       " skip   -  skip to the next message\n"
+       " delete -  delete this one and to the next message\n"
+       " count  -  count this message as processed\n"
+       "Each keyword can be prefixed with \"no\" to reverse its meaning\n"
+       "Setting onerror=abort reverts to the default behavior.") },
   { NULL }
 };
 
@@ -336,7 +403,6 @@ struct mu_cfg_param movemail_cfg_param[] = {
 static const char *movemail_capa[] = {
   "common",
   "debug",
-  "license",
   "locking",
   "mailbox",
   "auth",
@@ -440,22 +506,16 @@ open_mailbox (mu_mailbox_t *mbx, char *name, int flags, char *passwd)
 }
 
 int
-move_message (mu_mailbox_t src, mu_mailbox_t dst, size_t msgno)
+move_message (mu_mailbox_t dst, mu_message_t msg, size_t msgno)
 {
   int rc;
-  mu_message_t msg;
 
-  if ((rc = mu_mailbox_get_message (src, msgno, &msg)) != 0)
-    {
-      mu_error (_("cannot read message %lu: %s"),
-		(unsigned long) msgno, mu_strerror (rc));
-      return rc;
-    }
   if ((rc = mu_mailbox_append_message (dst, msg)) != 0)
     {
       mu_error (_("cannot append message %lu: %s"),
 		(unsigned long) msgno, mu_strerror (rc));
-      return rc;
+      if (!(onerror_flags & ONERROR_DELETE))
+	return rc;
     }
   if (!preserve_mail)
     {
@@ -466,6 +526,26 @@ move_message (mu_mailbox_t src, mu_mailbox_t dst, size_t msgno)
   return rc;
 }
 
+int
+movemail (mu_mailbox_t dst, mu_message_t msg, size_t msgno)
+{
+  int rc = move_message (dst, msg, msgno);
+  if (rc == 0)
+    ++msg_count;
+  else
+    {
+      app_err_count++;
+      if (onerror_flags)
+	{
+	  if (onerror_flags & ONERROR_COUNT)
+	    ++msg_count;
+	}
+      else
+	return 1;
+    }
+  return max_messages_option && msg_count >= max_messages_option;
+}
+
 /* Open source mailbox using compatibility syntax. Source_name is
    of the form:
 
@@ -487,7 +567,7 @@ compatibility_mode (mu_mailbox_t *mbx, char *source_name, char *password,
       mu_error (_("hostname of the POP3 server is unknown"));
       exit (1);
     }
-  asprintf (&tmp, "pop://%s@%s", user_name, host);
+  mu_asprintf (&tmp, "pop://%s@%s", user_name, host);
   open_mailbox (mbx, tmp, flags, password);
   free (tmp);
 }
@@ -607,6 +687,8 @@ guess_mbox_owner (mu_mailbox_t mbox, struct user_id *id)
 		  rc = 1;
 		}
 	    }
+	  else
+	    id->gid = meth->owner.id.gid;
 	  break;
 	  
 	case set_owner_name:
@@ -654,97 +736,126 @@ _compare_uidls (const void *item, const void *value)
 
   return strcmp (a->uidl, b->uidl);
 }
+
+struct movemail_getvar_closure
+{
+  const char *source_name;
+  const char *dest_name;
+  mu_url_t source_url;
+  mu_url_t dest_url;
+};
 
-#define __cat2__(a,b) a ## b
-#define DCL_VTX(what)						\
-static int							\
- __cat2__(_vtx_,what) (const char *name, void *data, char **p)	\
-{								\
-  mu_url_t url = data;			                        \
-  int rc = __cat2__(mu_url_aget_,what) (url, p);                \
-  if (rc == MU_ERR_NOENT)	                                \
-    {                                                           \
-      *p = strdup ("");						\
-      return 0;                                                 \
-    }                                                           \
-  return rc;                                                    \
+#define SEQ(s, n, l) \
+  (((l) == (sizeof(s) - 1)) && memcmp (s, n, l) == 0)
+
+static const char *
+get_url_part (mu_url_t url, const char *name, size_t nlen)
+{
+  int rc = MU_ERR_NOENT;
+  const char *s;
+  
+  if (!url)
+    return NULL;
+  if (SEQ ("user", name, nlen))
+    rc = mu_url_sget_user (url, &s);
+  else if (SEQ ("host", name, nlen))
+    rc = mu_url_sget_host (url, &s);
+  else if (SEQ ("port", name, nlen))
+    rc = mu_url_sget_portstr (url, &s);
+  else if (SEQ ("path", name, nlen))
+    rc = mu_url_sget_path (url, &s);
+
+  if (rc)
+    return NULL;
+  return s;
 }
+    
+static const char *
+movemail_getvar (const char *name, size_t nlen, void *data)
+{
+  struct movemail_getvar_closure *pc = data;
+  
+  if (SEQ ("progname", name, nlen))
+    return mu_program_name;
+  if (SEQ ("source", name, nlen))
+    return pc->source_name;
+  if (SEQ ("dest", name, nlen))
+    return pc->dest_name;
 
-DCL_VTX (host)
-DCL_VTX (user)
-DCL_VTX (path)
+  if (nlen > 7 && memcmp ("source_", name, 7) == 0)
+    return get_url_part (pc->source_url, name + 7, nlen - 7);
+  if (nlen > 5 && memcmp ("dest_", name, 5) == 0)
+    return get_url_part (pc->dest_url, name + 5, nlen - 5);
+  
+  return NULL;
+}
 
 static void
 set_program_id (const char *source_name, const char *dest_name)
 {
   int rc;
-  mu_vartab_t vtab;
-  char *id;
-  mu_url_t url;
-      
-  mu_vartab_create (&vtab);
-  mu_vartab_define (vtab, "progname", mu_program_name, 1);
-  mu_vartab_define (vtab, "source", source_name, 1);
-  rc = mu_mailbox_get_url (source, &url);
+  struct mu_wordsplit ws;
+  struct movemail_getvar_closure clos;
+
+  clos.source_name = source_name;
+  clos.dest_name = dest_name;
+  rc = mu_mailbox_get_url (source, &clos.source_url);
   if (rc)
     mu_diag_output (MU_DIAG_INFO,
 		    _("cannot obtain source mailbox URL: %s"),
 		    mu_strerror (rc));
-  else
-    {
-      mu_vartab_define_exp (vtab, "source:user", _vtx_user, NULL, url);
-      mu_vartab_define_exp (vtab, "source:host", _vtx_host, NULL, url);
-      mu_vartab_define_exp (vtab, "source:path", _vtx_path, NULL, url);
-    }
-      
-  mu_vartab_define (vtab, "dest", dest_name, 1);
-  rc = mu_mailbox_get_url (dest, &url);
+  rc = mu_mailbox_get_url (dest, &clos.dest_url);
   if (rc)
     mu_diag_output (MU_DIAG_INFO,
 		    _("cannot obtain destination mailbox URL: %s"),
 		    mu_strerror (rc));
-  else
+  
+  ws.ws_getvar = movemail_getvar;
+  ws.ws_closure = &clos;
+  if (mu_wordsplit (program_id_option, &ws,
+		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
+		    MU_WRDSF_GETVAR | MU_WRDSF_CLOSURE))
     {
-      mu_vartab_define_exp (vtab, "dest:user", _vtx_user, NULL, url);
-      mu_vartab_define_exp (vtab, "dest:host", _vtx_host, NULL, url);
-      mu_vartab_define_exp (vtab, "dest:path", _vtx_path, NULL, url);
+      mu_error (_("cannot expand line `%s': %s"), program_id_option,
+		mu_wordsplit_strerror (&ws));
+      return;
     }
-      
-  rc = mu_vartab_expand (vtab, program_id_option, &id);
-  mu_vartab_destroy (&vtab);
-  /*      asprintf (&id, "%s: %s", mu_program_name, s);
-	  free (s);*/
+  
   /* FIXME: Don't use mu_set_program_name here, because it
      plays wise with its argument. We need a mu_set_diag_prefix
      function. */
-  mu_program_name = id;
+  mu_program_name = ws.ws_wordv[0];
+  ws.ws_wordv[0] = NULL;
+  ws.ws_wordc = 0;
+  mu_wordsplit_free (&ws);
+  mu_stdstream_strerr_setup (MU_STRERR_STDERR);
 }
 
+
 int
 main (int argc, char **argv)
 {
   int index;
-  size_t i, total;
+  size_t total;
   int rc = 0;
-  int errs = 0;
   char *source_name, *dest_name;
   int flags;
   mu_list_t src_uidl_list = NULL;
-  size_t msg_count = 0;
+  mu_iterator_t itr;
   
   /* Native Language Support */
   MU_APP_INIT_NLS ();
   MU_AUTH_REGISTER_ALL_MODULES ();
   
-  /* Register the desired mailbox formats.  */
-  mu_register_all_mbox_formats ();
+  /* Register the desired "mailbox" formats.  */
+  mu_register_all_formats ();
 
   /* argument parsing */
   
 #ifdef WITH_TLS
   mu_gocs_register ("tls", mu_tls_module_init);
 #endif
-  mu_argp_init (program_version, NULL);
+  mu_argp_init (NULL, NULL);
   if (mu_app_init (&argp, movemail_capa, movemail_cfg_param, 
 		   argc, argv, 0, &index, NULL))
     exit (1);
@@ -758,6 +869,17 @@ main (int argc, char **argv)
       return 1;
     }
 
+  if (ignore_errors)
+    onerror_flags |= ONERROR_SKIP|ONERROR_COUNT;
+  
+  if (emacs_mode)
+    {      
+      /* Undo the effect of configuration options that may affect
+	 interaction with Emacs. */
+      mu_registrar_set_default_record (mu_mbox_record);
+      mu_stdstream_strerr_setup (MU_STRERR_STDERR);
+    }
+  
   atexit (close_mailboxes);
   
   source_name = argv[0];
@@ -785,13 +907,24 @@ main (int argc, char **argv)
     }
   
   if (verbose_option)
-    mu_diag_output (MU_DIAG_INFO,
-		    _("number of messages in source mailbox: %lu"),
-		    (unsigned long) total);
-
+    {
+      mu_diag_output (MU_DIAG_INFO,
+		      _("number of messages in source mailbox: %lu"),
+		      (unsigned long) total);
+      if (max_messages_option)
+	mu_diag_output (MU_DIAG_INFO,
+			reverse_order ?
+			ngettext ("will process last %lu message",
+				  "will process last %lu messages",
+				  max_messages_option) :
+			ngettext ("will process first %lu message",
+				  "will process first %lu messages",
+				  max_messages_option),
+			(unsigned long) max_messages_option);
+    }
+  
   if (uidl_option)
     {
-      mu_iterator_t itr;
       mu_list_t dst_uidl_list = NULL;
 
       rc = mu_mailbox_get_uidls (source, &src_uidl_list);
@@ -813,90 +946,107 @@ main (int argc, char **argv)
 	  if (mu_list_locate (dst_uidl_list, uidl, NULL) == 0)
 	    mu_iterator_ctl (itr, mu_itrctl_delete, NULL);
 	}
-      mu_iterator_destroy (&itr);
       mu_list_destroy (&dst_uidl_list);
       mu_list_set_comparator (src_uidl_list, NULL);
-    }
 
-  /* FIXME: Implementing a mailbox iterator would allow to merge the three
-     branches of this conditional. */
-  if (src_uidl_list)
-    {
-      mu_iterator_t itr;
-
-      rc = mu_list_get_iterator (src_uidl_list, &itr);
-      if (rc)
-	{
-	  mu_error(_("cannot get iterator: %s"), mu_strerror (rc));
-	  exit (1);
-	}
       rc = mu_iterator_ctl (itr, mu_itrctl_set_direction, &reverse_order);
       if (rc)
 	{
-	  mu_error(_("cannot set iteration direction: %s"), mu_strerror (rc));
+	  mu_error (_("cannot set iteration direction: %s"), mu_strerror (rc));
 	  exit (1);
 	}
+      
       for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
 	   mu_iterator_next (itr))
 	{
 	  struct mu_uidl *uidl;
-	      
+	  mu_message_t msg;
+	  
 	  mu_iterator_current (itr, (void **)&uidl);
-	  rc = move_message (source, dest, uidl->msgno);
-	  if (rc == 0)
-	    msg_count++;
-	  else if (!ignore_errors)
+
+	  if ((rc = mu_mailbox_get_message (source, uidl->msgno, &msg)) != 0)
+	    {
+	      mu_error (_("cannot read message %lu: %s"),
+			(unsigned long) uidl->msgno, mu_strerror (rc));
+	      get_err_count++;
+	      continue;
+	    }
+	  if (movemail (dest, msg, uidl->msgno))
 	    break;
-	  else
-	    errs = 1;
-	}
-      mu_iterator_destroy (&itr);
-    }
-  else if (reverse_order)
-    {
-      for (i = total; i > 0; i--)
-	{
-	  rc = move_message (source, dest, i);
-	  if (rc == 0)
-	    msg_count++;
-	  else if (!ignore_errors)
-	    break;
-	  else
-	    errs = 1;
 	}
     }
   else
     {
-      for (i = 1; i <= total; i++)
+      rc = mu_mailbox_get_iterator (source, &itr);
+      if (rc)
 	{
-	  rc = move_message (source, dest, i);
-	  if (rc == 0)
-	    msg_count++;
-	  else if (!ignore_errors)
+	  mu_error (_("cannot obtain mailbox iterator: %s"), mu_strerror (rc));
+	  return 1;
+	}
+  
+      rc = mu_iterator_ctl (itr, mu_itrctl_set_direction, &reverse_order);
+      if (rc)
+	{
+	  mu_error (_("cannot set iteration direction: %s"),
+		    mu_strerror (rc));
+	  return 1;
+	}
+      
+      for (mu_iterator_first (itr); !mu_iterator_is_done (itr);
+	   mu_iterator_next (itr))
+	{
+	  mu_message_t msg;
+	  size_t msgno;
+	  
+	  rc = mu_iterator_ctl (itr, mu_itrctl_tell, &msgno);
+	  if (rc)
+	    {
+	      mu_error (_("cannot get iterator position: %s"),
+			mu_strerror (rc));
+	      return 1;
+	    }
+
+	  rc = mu_iterator_current (itr, (void **)&msg);
+	  if (rc)
+	    {
+	      mu_error (_("cannot read message %lu: %s"),
+			(unsigned long) msgno, mu_strerror (rc));
+	      get_err_count++;
+	      continue;
+	    }
+	  
+	  if (movemail (dest, msg, msgno))
 	    break;
-	  else
-	    errs = 1;
 	}
     }
+  mu_iterator_destroy (&itr);
   
   if (verbose_option)
-    mu_diag_output (MU_DIAG_INFO,
-		    _("number of processed messages: %lu"),
-		    (unsigned long) msg_count);
-  
-  if (rc)
-    return !!rc;
+    {
+      mu_diag_output (MU_DIAG_INFO,
+		      _("number of processed messages: %lu"),
+		      (unsigned long) msg_count);
+      mu_diag_output (MU_DIAG_INFO,
+		      _("number of errors: %lu / %lu"),
+		      (unsigned long) get_err_count,
+		      (unsigned long) app_err_count);
+    }
   
+  if (app_err_count && !(onerror_flags & (ONERROR_DELETE|ONERROR_COUNT)))
+    preserve_mail = 1;
+  if (onerror_flags & ONERROR_COUNT)
+    app_err_count = 0;
+      
   mu_mailbox_sync (dest);
   rc = mu_mailbox_close (dest);
   mu_mailbox_destroy (&dest);
   if (rc)
     mu_error (_("cannot close destination mailbox: %s"), mu_strerror (rc));
-  else
-    mu_mailbox_flush (source, 1);
+  else if (!preserve_mail)
+    mu_mailbox_expunge (source);
 
   mu_mailbox_close (source);
   mu_mailbox_destroy (&source);
-
-  return !(rc == 0 && errs == 0);
+  
+  return !(rc == 0 && (app_err_count + get_err_count) == 0);
 }
