@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 
-   2009 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2002, 2003, 2005, 2007, 2009, 2010, 2011
+   Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -22,7 +22,8 @@ struct mu_auth_data *auth_data;
 int
 pop3d_begin_session ()
 {
-  int status;
+  mu_url_t url = NULL;
+  size_t total = 0;
 
   mu_diag_output (MU_DIAG_INFO, _("POP3 login: user `%s', source %s"),
 		  auth_data->name, auth_data->source);
@@ -40,19 +41,9 @@ pop3d_begin_session ()
   if (auth_data->change_uid)
     setuid (auth_data->uid);
   
-  if ((status = mu_mailbox_create (&mbox, auth_data->mailbox)) != 0
-      || (status = mu_mailbox_open (mbox, MU_STREAM_CREAT | MU_STREAM_RDWR)) != 0)
+  if (manlock_open_mailbox (&mbox, auth_data->mailbox, 0,
+			    MU_STREAM_CREAT | MU_STREAM_RDWR))
     {
-      mu_mailbox_destroy (&mbox);
-      state = AUTHORIZATION;
-      mu_auth_data_destroy (&auth_data);
-      return ERR_MBOX_LOCK;
-    }
-  
-  if (pop3d_lock ())
-    {
-      mu_mailbox_close (mbox);
-      mu_mailbox_destroy (&mbox); 
       mu_auth_data_destroy (&auth_data);
       state = AUTHORIZATION;
       return ERR_MBOX_LOCK;
@@ -63,25 +54,21 @@ pop3d_begin_session ()
     pop3d_abquit (ERR_NO_MEM);
   state = TRANSACTION;
 
-  pop3d_outf ("+OK opened mailbox for %s\r\n", username);
+  pop3d_outf ("+OK opened mailbox for %s\n", username);
 
   if (undelete_on_startup)
     pop3d_undelete_all ();
 
   deliver_pending_bulletins ();
   
-  /* mailbox name */
-  {
-    mu_url_t url = NULL;
-    size_t total = 0;
-    mu_mailbox_get_url (mbox, &url);
-    mu_mailbox_messages_count (mbox, &total);
-    mu_diag_output (MU_DIAG_INFO,
+  /* log mailbox stats */
+  mu_mailbox_get_url (mbox, &url);
+  mu_mailbox_messages_count (mbox, &total);
+  mu_diag_output (MU_DIAG_INFO,
 	    ngettext ("user `%s' logged in with mailbox `%s' (%s message)",
 		      "user `%s' logged in with mailbox `%s' (%s messages)",
 		      (unsigned long) total),
 	    username, mu_url_to_string (url), mu_umaxtostr (0, total));
-  }
 
   return OK;
 }
@@ -89,8 +76,9 @@ pop3d_begin_session ()
 int
 pop3d_user (char *arg)
 {
-  char *buf, pass[POP_MAXCMDLEN], *tmp, *cmd;
+  char *buf, *pass, *cmd;
   char buffer[512];
+  int xscript_level;
   
   if (state != AUTHORIZATION)
     return ERR_WRONG_STATE;
@@ -98,21 +86,14 @@ pop3d_user (char *arg)
   if ((strlen (arg) == 0) || (strchr (arg, ' ') != NULL))
     return ERR_BAD_ARGS;
 
-  pop3d_outf ("+OK\r\n");
+  pop3d_outf ("+OK\n");
   pop3d_flush_output ();
 
+  xscript_level = set_xscript_level (MU_XSCRIPT_SECURE);
   buf = pop3d_readline (buffer, sizeof (buffer));
-  pop3d_parse_command (buf, &cmd, &tmp);
-
-  if (strlen (tmp) > POP_MAXCMDLEN)
-    return ERR_TOO_LONG;
-  else
-    {
-      strncpy (pass, tmp, POP_MAXCMDLEN);
-      /* strncpy () is lame, make sure the string is null terminated.  */
-      pass[POP_MAXCMDLEN - 1] = '\0';
-    }
-
+  pop3d_parse_command (buf, &cmd, &pass);
+  set_xscript_level (xscript_level);
+  
   if (mu_c_strcasecmp (cmd, "PASS") == 0)
     {
       int rc;
@@ -122,7 +103,8 @@ pop3d_user (char *arg)
       tmp = pop3d_apopuser (arg);
       if (tmp != NULL)
 	{
-	  mu_diag_output (MU_DIAG_INFO, _("APOP user %s tried to log in with USER"), arg);
+	  mu_diag_output (MU_DIAG_INFO,
+			  _("APOP user %s tried to log in with USER"), arg);
 	  return ERR_BAD_LOGIN;
 	}
 #endif
